from . import geometry
import yaml


class Phantom:
    def __init__(self,name):
        self.name = name
        self.geoList = []
        self.phantomSDList = []

    def addGeo(self,item):
        self.geoList.append(item)
    def addPhantomSD(self,item):
        self.phantomSDList.append(item.name)
    def getMacStr(self):
        mac =""
        for item in self.geoList:
            mac += item.getMacStr()
        mac += self.getSDMacStr()

        return mac
    def getSDMacStr(self):
        mac = ""
        fmt = r"/gate/{0}/attachPhantomSD" + "\n"
        for item in self.phantomSDList:
            mac += fmt.format(item)
        return mac



# if __name__ == '__main__':
#     c1  = geometry.Cylinder(mother = 'world', name = 'NemaCylinder', Rmax = 82, Rmin = 56, Height = 5)
#     # print (b1.getMacStr())
#     b1 = geometry.Box(mother = c1.name, name = 'block', position = geometry.Vec3(66.5, 0.0, 0.0), size = geometry.Vec3(20,44,5))
#     c1.addChild(b1)

#     phantom = Phantom(name = 'phantom1')
#     phantom.addGeo(c1)
#     phantom.addPhantomSD(b1)

#     # print(phantom.getMacStr())

# with open('phantom.yml', 'w') as fout:
#     yaml.dump(phantom, fout)



from . import geometry
import yaml
from .digitizer import AttrPair


class Camera:
    def __init__(self, name, system):
        self.name = name
        self.system = system
        self.geoList = []
        self.crystalSDList = []

    def addGeo(self, item):
        self.geoList.append(item)

    def addCrystalSD(self, item):
        self.crystalSDList.append(item)

    def getMacStr(self):
        mac = ""
        for item in self.geoList:
            mac += item.getMacStr()
        mac += self.system.getMacStr()
        mac += self.getSDMacStr()

        return mac

    def getSDMacStr(self):
        mac = ""
        fmt = r"/gate/{0}/attachCrystalSD" + "\n"
        for item in self.crystalSDList:
            mac += fmt.format(item)
        return mac


class Systems:
    sysList = ["PETscanner", "cylindricalPET", "ecat", "MultiPatchPET"]

    def __init__(self, name):
        if name in Systems.sysList:
            self.name = name
        else:
            print("invalid system name!  Systems:__init__() \n", name)
        self.levelList = []
        self.attachList = []

    def attachSystem(self, itemList):
        self.attachList = itemList

    def getMacStr(self):
        mac = ""
        fmt = r"/gate/systems/{0}/{1}/attach {2}" + "\n"
        # for i in range(len(self.attachList)):
        for l, a in zip(self.levelList, self.attachList):
            # mac += fmt.format(self.name, self.levelList[i], self.attachList[i])
            if a == 'void':
                mac = mac
            else:
                mac += fmt.format(self.name, l, a)
        return mac


class PETscanner(Systems):
    # levelList = ['level{0}'.format(i) for i in range(1, 6)]

    def __init__(self):
        super(PETscanner, self).__init__(name='PETscaner')
        self.levelList.append('level1')
        self.levelList.append('level2')
        self.levelList.append('level3')
        self.levelList.append('level4')
        self.levelList.append('level5')


class Ecat(Systems):
    # levelList = ['block', 'crystal']

    def __init__(self):
        super(Ecat, self).__init__(name='ecat')
        self.levelList.append('block')
        self.levelList.append('crystal')


class CylindericalPET(Systems):
    def __init__(self):
        super(CylindericalPET, self).__init__(name='cylindricalPET')
        self.levelList.append('rsector')
        self.levelList.append('module')
        self.levelList.append('submodule')
        self.levelList.append('crystal')
        self.levelList.append('layer0')
        self.levelList.append('layer1')
        self.levelList.append('layer2')
        self.levelList.append('layer3')


# if __name__ == '__main__':
#     #   an Ecat scanner case

#     # print (camera1.getMacStr())
#     with open('camera.yml', 'w') as fout:
#         yaml.dump(camera1, fout)



# _*_ coding: utf-8 _*_
from math import pi
from .digitizer import AttrPair
import yaml


class Vec3:
    def __init__(self, x, y, z):
        self.x = x
        self.y = y
        self.z = z

    def getMacStr(self):
        fmt1 = r' {0}    {1}    {2}'
        return fmt1.format(self.x, self.y, self.z)


class Volume:
    shapeList = ['box', 'cylinder', 'sphere',
                 'patch', 'ImageRegularParametrisedVolume']

    def __init__(self, shapeType, name, material=None, mother=None, position=None):
        if shapeType in Volume.shapeList:
            self.shapeType = shapeType
        else:
            print("invalid shape type: %s \n", shapeType)
        if mother is None:
            self.mother = 'world'
        else:
            self.mother = mother
        self.material = material

        self.name = name
        self.position = position
        self.attrList = []
        self.childList = []

    def makeAttrList(self):
        nameFmt = r"/gate/{0}/daughters/name {1}" + "\n"
        insertFmt = r"/gate/{0}/daughters/insert {1}" + "\n"
        posFmt =   r"/gate/{0}/placement/setTranslation {1}  mm" + "\n"
        matFmt =  r"/gate/{0}/setMaterial {1}" + "\n"
        self.addAttr(
            AttrPair(self.mother, nameFmt.format(self.mother, self.name)))
        self.addAttr(AttrPair(self.mother, insertFmt.format(
            self.mother, self.shapeType)))
        if self.position is not None:
            self.addAttr(
                AttrPair(self.position, posFmt.format(self.name,self.position.getMacStr())))
        if self.material is not None:
            self.addAttr(AttrPair(self.material, matFmt.format(self.name,self.material)))

    def addChild(self, child):
        child.mother = self.name
        self.childList.append(child)

    def addAttr(self, attrItem):
        self.attrList.append(attrItem)

    def getMacStr(self):
        self.makeAttrList()
        mac = ""
        for item in self.attrList:
            if item.value is not None:
                mac += item.fmtstr
        for item in self.childList:
            mac += item.getMacStr()
        return mac

    def getMeStr(self):
        fmt = r"/gate/{0}/geometry"
        return fmt.format(self.name)


class Box(Volume):
    def __init__(self,  name, mother=None, position=None, material=None, size=None):
        super(Box, self).__init__(shapeType='box', mother=mother,
                                  name=name, position=position, material=material)
        if size is None:
            print("no box size in Box: %s", self.name)
        else:
            self.size = size

    def makeAttrList(self):
        super(Box, self).makeAttrList()
        sizeFmt = self.getMeStr() + r"/setXLength {0} mm" + "\n" + self.getMeStr(
        ) + r"/setYLength {1} mm" + "\n" + self.getMeStr() + r"/setZLength {2} mm" + "\n"
        self.addAttr(AttrPair(self.size, sizeFmt.format(
            self.size.x, self.size.y, self.size.z)))


class Cylinder(Volume):
    def __init__(self, name, Rmax, mother=None, Height=None, position=None, material=None,
                 Rmin=None, PhiStart=None, DeltaPhi=None):
        super(Cylinder, self).__init__(shapeType='cylinder',
                                       mother=mother, name=name, position=position, material=material)
        self.Rmin = Rmin
        self.Rmax = Rmax
        self.Height = Height
        self.PhiStart = PhiStart
        self.DeltaPhi = DeltaPhi

    def makeAttrList(self):
        super(Cylinder, self).makeAttrList()
        fmtRmin = self.getMeStr() + r"/setRmin {0} mm" + "\n"
        fmtRmax = self.getMeStr() + r"/setRmax {0} mm" + "\n"
        fmtHeight = self.getMeStr() + r"/setHeight {0} mm" + "\n"
        fmtPhiStart = self.getMeStr() + r"/setPhiStart {0}  deg" + "\n"
        fmtDeltaPhi = self.getMeStr() + r"/setDeltaPhi {0}  deg" + "\n"

        self.addAttr(AttrPair(self.Rmin, fmtRmin.format(self.Rmin)))
        self.addAttr(AttrPair(self.Rmax, fmtRmax.format(self.Rmax)))
        self.addAttr(AttrPair(self.Height, fmtHeight.format(self.Height)))
        self.addAttr(
            AttrPair(self.PhiStart, fmtPhiStart.format(self.PhiStart)))
        self.addAttr(
            AttrPair(self.DeltaPhi, fmtDeltaPhi.format(self.DeltaPhi)))


class Sphere(Volume):
    def __init__(self, name, Rmax, mother=None, position=None, material=None,
                 Rmin=None, PhiStart=None, DeltaPhi=None, ThetaStart=None, DeltaTheta=None):
        super(Sphere, self).__init__(shapeType='sphere',
                                     mother=mother, name=name, position=position, material=material)
        self.Rmin = Rmin
        self.Rmax = Rmax
        self.PhiStart = PhiStart
        self.DeltaPhi = DeltaPhi
        self.ThetaStart = ThetaStart
        self.DeltaTheta = DeltaTheta

    def makeAttrList(self):
        super(Sphere, self).makeAttrList()
        fmtRmin = self.getMeStr() + r"/setRmin {0} mm" + "\n"
        fmtRmax = self.getMeStr() + r"/setRmax {0} mm" + "\n"
        fmtPhiStart = self.getMeStr() + r"/setPhiStart {0}  deg" + "\n"
        fmtDeltaPhi = self.getMeStr() + r"/setDeltaPhi {0}  deg" + "\n"
        fmtThetaStart = self.getMeStr() + r"/setThetaStart {0}  deg" + "\n"
        fmtDeltaTheta = self.getMeStr() + r"/setDeltaTheta {0}  deg" + "\n"

        self.addAttr(AttrPair(self.Rmin, fmtRmin.format(self.Rmin)))
        self.addAttr(AttrPair(self.Rmax, fmtRmax.format(self.Rmax)))
        self.addAttr(
            AttrPair(self.PhiStart, fmtPhiStart.format(self.PhiStart)))
        self.addAttr(
            AttrPair(self.DeltaPhi, fmtDeltaPhi.format(self.DeltaPhi)))
        self.addAttr(AttrPair(self.ThetaStart,
                              fmtThetaStart.format(self.ThetaStart)))
        self.addAttr(AttrPair(self.DeltaTheta,
                              fmtDeltaTheta.format(self.DeltaTheta)))


class ImageRegularParamerisedVolume(Volume):
    def __init__(self,  name, imagefile, rangefile,  mother=None, position=None, material=None):
        super(ImageRegularParamerisedVolume, self).__init__(shapeType='ImageRegularParametrisedVolume', mother=mother,
                                                            name=name, position=position, material=None)
        self.imagefile = imagefile
        self.rangefile = rangefile

    def makeAttrList(self):
        super(ImageRegularParamerisedVolume, self).makeAttrList()
        ImageFileFmt = self.getMeStr() + r"/setImage {0} "+ "\n"
        self.addAttr(
            AttrPair(self.imagefile, ImageFileFmt.format(self.imagefile)))
        RangeFileFmt = self.getMeStr() + r"/setRangeToMaterialFile {0} "+"\n"
        self.addAttr(
            AttrPair(self.rangefile, RangeFileFmt.format(self.rangefile)))


class Repeater:
    def __init__(self, volume):
        self.volume = volume

    def getMacStr(self):
        pass


class RingRepeater(Repeater):
    def __init__(self, volume, number):
        super(RingRepeater, self).__init__(volume)
        self.number = number

    def getMacStr(self):
        fmt1 = r"/gate/{0}/repeaters/insert ring " + "\n"
        fmt2 = r"/gate/{0}/ring/setRepeatNumber {1}" + "\n"

        return(fmt1.format(self.volume)
               + fmt2.format(self.volume, self.number))


class LinearRepeater(Repeater):
    def __init__(self, volume, number, repeatVector):
        super(LinearRepeater, self).__init__(volume)
        self.number = number
        self.repeatVector = repeatVector

    def getMacStr(self):
        fmt1 = r"/gate/{0}/repeaters/insert linear " + "\n"
        fmt2 = r"/gate/{0}/linear/setRepeatNumber {1}" + " \n"
        fmt3 = r"/gate/{0}/linear/setRepeatVector {1}" + " \n"
        return (fmt1.format(self.volume) +
                fmt2.format(self.volume, self.number) +
                fmt3.format(self.volume, self.repeatVector))


class CubicRepeater(Repeater):
    def __init__(self, volume, scale, repeatVector):
        super(CubicRepeater, self).__init__(volume)
        self.scale = scale
        self.repeatVector = repeatVector

    def getMacStr(self):
        fmt1 = r"/gate/{0}/repeaters/insert cubicArray" + "\n"
        fmt2 = r"/gate/{0}/cubicArray/setRepeatNumberX   {1}" + " \n"
        fmt3 = r"/gate/{0}/cubicArray/setRepeatNumberY   {1}" + " \n"
        fmt4 = r"/gate/{0}/cubicArray/setRepeatNumberZ   {1}" + " \n"
        fmt5 = r"/gate/{0}/cubicArray/setRepeatVector {1}  mm" + " \n"

        return (fmt1.format(self.volume) +
                fmt2.format(self.volume, self.scale.x) +
                fmt3.format(self.volume, self.scale.y) +
                fmt4.format(self.volume, self.scale.z) +
                fmt5.format(self.volume, self.repeatVector.getMacStr())
                )


# if __name__ == '__main__':
#     c1 = Cylinder(mother='world', name='c1', Rmax = 30, Height = 30)
#     b1 = Box(mother = c1.name, name = 'b1', size=  Vec3(5,5,5))
#     rr = RingRepeater(volume = b1.name,number = 10)
#     c1.addChild(b1)

#     b2 = Box(mother = b1.name, name = 'b2',size = Vec3(2,2,2))
#     cr = CubicRepeater(volume = b2.name,scale = Vec3(2,2,2), repeatVector = Vec3(2.1,2.1,2.1))

#     b1.addChild(b2)


#     print(c1.getMacStr())
#     with open('geo.yml', 'w') as fout:
#         yaml.dump(b1, fout)



from . import camera
from . import phantom 
from . import source
from . import digitizer
from . import physics
from . import geometry

from fs.osfs import OSFS
import os
import yaml

class RunTime:
    def __init__(self,  endTime=1, timeSlice=1, startTime=0):
        self.endTime = endTime
        self.timeSlice = timeSlice
        self.startTime = startTime

    def getMacStr(self):
        fmt = (r"/gate/application/setTimeSlice  {0} s" + "\n"
               + r"/gate/application/setTimeStart  {1} s" + "\n"
               + r"/gate/application/setTimeStop  {2} s" + "\n")
        return fmt.format(self.timeSlice, self.startTime, self.endTime)


class DataOut:
    OutList = ["ascii", "binary", "root", "sinogram"]

    def __init__(self, outType, fileName):
        self.outType = outType
        self.fileName = fileName

    def getMacStr(self):
        fmt = (r"/gate/output/{0}/enable" + "\n"
               + r"/gate/output/{0}/setFileName  {1}" + "\n")
        return fmt.format(self.outType, self.fileName)


class FlagPair:
    def __init__(self, name, value):
        self.name = name
        self.value = value


class Root(DataOut):
    def __init__(self, fileName, flagList=[FlagPair('Hit', 0), FlagPair('Singles', 1), FlagPair('Coincidences', 1)]):
        super(Root, self).__init__(outType='root', fileName=fileName)
        self.flagList = flagList

    def addFlag(self, item):
        self.flagList.append(item)

    def getMacStr(self):
        mac = ""
        mac += super(Root, self).getMacStr()
        fmt = r"/gate/output/root/setRoot{0}Flag    {1}" + "\n"
        for item in self.flagList:
            mac += fmt.format(item.name, item.value)
        return mac


class Sino(DataOut):
    def __init__(self, fileName, inputDataName, tangCrystalBlurring=1.8, axialCrystalBlurring=1.8, radialBins=None, rawFlag='true', delayFlag=1, scatterFlag=1):
        super(Sino, self).__init__(outType='sinogram', fileName=fileName)
        self.inputDataName = inputDataName
        self.tangCrystalBlurring = tangCrystalBlurring
        self.axialCrystalBlurring = axialCrystalBlurring
        self.radialBins = radialBins
        self.rawFlag = rawFlag
        self.delayFlag = delayFlag
        self.scatterFlag = scatterFlag

    def getMacStr(self):
        mac = ''
        mac += super(Sino, self).getMacStr()
        fmt1 = r"/gate/output/sinogram/setTangCrystalBlurring {0} mm" + "\n"
        fmt2 = r"/gate/output/sinogram/setAxialCrystalBlurring {0} mm" + "\n"
        fmt3 = r"/gate/output/sinogram/RawOutputEnable true" + "\n"
        fmt4 = r"/gate/output/sinogram/StoreDelayeds" + "\n"
        fmt5 = r"/gate/output/sinogram/StoreScatters" + "\n"
        fmt6 = r"/gate/output/sinogram/setInputDataName {0}" + "\n"
        if self.tangCrystalBlurring is not None:
            mac += fmt1.format(self.tangCrystalBlurring)
        if self.axialCrystalBlurring is not None:
            mac += fmt2.format(self.axialCrystalBlurring)
        mac += fmt3.format()
        if self.delayFlag is not None:
            mac += fmt4.format()
        if self.scatterFlag is not None:
            mac += fmt5.format()
        mac += fmt6.format(self.inputDataName)
        return mac


class RandomEngine:
    engineList = ['JamesRandom', 'Ranlux64', 'MersenneTwister']

    def __init__(self, name='JamesRandom', seed='default'):
        self.name = name
        self.seed = seed

    def getMacStr(self):
        fmt = (r"/gate/random/setEngineName {0}" + "\n"
               + r"/gate/random/setEngineSeed {1}" + "\n")
        return fmt.format(self.name, self.seed)


class SimuApp:
    def __init__(self, name='simu1', cam=None, phan=None, src=None, digi=None, phy=None,
                 runTime=RunTime(), worldSize=geometry.Vec3(100, 100, 100), randEngine=RandomEngine()):
        self.name = name
        self.cam = cam
        self.phan = phan
        self.src = src
        self.digi = digi
        self.phy = phy
        self.dataOutList = []
        self.runTime = runTime
        self.worldSize = worldSize
        self.randEngine = randEngine

    def addDataout(self, item):
        if item is not None:
            self.dataOutList.append(item)

    def getComMac(self):
        mac = ""
        camFmt = r"/control/execute camera.mac" + "\n"
        phanFmt = r"/control/execute phantom.mac" + "\n"
        phyFmt = r"/control/execute physics.mac" + "\n"
        initFmt = r"/gate/run/initialize" + "\n"
        digiFmt = r"/control/execute digitizer.mac" + "\n"
        srcFmt = r"/control/execute source.mac" + "\n"
        mac = camFmt + phanFmt + phyFmt + initFmt + digiFmt + srcFmt
        return mac

    def getMacStr(self):
        mac = ""
        materdbFmt = r"/gate/geometry/setMaterialDatabase    ./GateMaterials.db" + "\n"
        worldFmt = (r"/gate/world/geometry/setXLength  {0} cm" + "\n"
                    + r"/gate/world/geometry/setYLength  {1} cm" + "\n"
                    + r"/gate/world/geometry/setZLength  {2} cm" + "\n")

        startFmt = r"/gate/application/startDAQ" + "\n"

        mac = (materdbFmt + worldFmt.format(self.worldSize.x, self.worldSize.y, self.worldSize.z)
               + self.getComMac() + self.randEngine.getMacStr())
        for item in self.dataOutList:
            mac += item.getMacStr()
        mac += self.runTime.getMacStr() + startFmt
        return mac

    def generateYaml(self):
        with open(self.name + '.yml', 'w') as fout:
            yaml.dump(self, fout)

    def generateMacs(self):
        # currPath = os.getcwd()
        # if(os.path.exists("SimuMacs")):
        #     with OSFS('.') as fs:
        #         fs.removetree('SimuMacs')
                # os.rmdir('SimuMacs')
        # else:
        # if(not os.path.exists("SimuMacs")):
        #     os.mkdir('SimuMacs')
        # with open(os.getcwd() + '/SimuMacs/camera.mac', 'w') as file_object:
        #     file_object.write(self.cam.getMacStr())
        # file_object.close()
        subroot = ''
        # subroot = '/SimuMacs'
        file_object = open(os.getcwd() + subroot + '/camera.mac', 'w')
        file_object.write(self.cam.getMacStr())
        file_object.close()

        file_object = open(os.getcwd() + subroot + '/phantom.mac', 'w')
        file_object.write(self.phan.getMacStr())
        file_object.close()

        file_object = open(os.getcwd() + subroot + '/physics.mac', 'w')
        file_object.write(self.phy.getMacStr())
        file_object.close()

        file_object = open(os.getcwd() + subroot + '/digitizer.mac', 'w')
        file_object.write(self.digi.getMacStr())
        file_object.close()

        file_object = open(os.getcwd() + subroot + '/source.mac', 'w')
        file_object.write(self.src.getMacStr())
        file_object.close()

        file_object = open(os.getcwd() + subroot + '/main.mac', 'w')
        file_object.write(self.getMacStr())
        file_object.close()


# if __name__ == '__main__':


class MacMaker:
    @classmethod
    def make_mac(cls, config):
        if not isinstance(config, (list, tuple)):
            config = [config]
        for f in config:
            with open(f) as fin:
                simu = yaml.load(fin)
                simu.generateMacs()

    @classmethod
    def make_yml(cls, yml_filename):
        # Camera
        # c1 = geometry.Cylinder(name='ecat', Rmax=85,
        #                        Rmin=59, Height=10, material='Air')
        # # print (b1.getMacStr())
        # b1 = geometry.Box(name='block', position=geometry.Vec3(
        #     69.0, 0.0, 0.0), size=geometry.Vec3(20, 30, 3), material='Air')
        # b2 = geometry.Box(name='crystal', size=geometry.Vec3(
        #     20, 1, 1), material='LYSO')
        # c1.addChild(b1)
        # b1.addChild(b2)
        # cbr1 = geometry.RingRepeater(volume=b1.name, number=10)
        # cbr2 = geometry.CubicRepeater(volume=b2.name, scale=geometry.Vec3(
        #     1, 30, 3), repeatVector=geometry.Vec3(0, 1, 1))
        # sys = camera.Ecat()
        # sys.attachSystem(itemList=[b1.name, b2.name])

        # # create the camera and construt it
        # camera1 = camera.Camera(name='cam1', system=sys)
        # camera1.addGeo(c1)
        # camera1.addGeo(cbr2)
        # camera1.addGeo(cbr1)
        # camera1.addCrystalSD(b2.name)

        # HRRT
        hrrt_c1 = geometry.Cylinder(
            name='cylindricalPET', Rmax=254.5, Rmin=234.5, Height=253.5, material='Air')
        hrrt_head = geometry.Box(name='block', position=geometry.Vec3(
            244.5, 0, 0), size=geometry.Vec3(20, 175.68, 253.5), material='Air')
        hrrt_module = geometry.Box(name='module', position=geometry.Vec3(
            0, 0, 0), size=geometry.Vec3(20, 19, 19), material='Air')
        hrrt_crystal = geometry.Box(name='crystal', position=geometry.Vec3(
            0, 0, 0), size=geometry.Vec3(20, 2, 2), material='LYSO')

        hrrt_c1.addChild(hrrt_head)
        hrrt_head.addChild(hrrt_module)
        hrrt_module.addChild(hrrt_crystal)

        hrrt_cbr2 = geometry.CubicRepeater(volume=hrrt_crystal.name, scale=geometry.Vec3(
            1, 8, 8), repeatVector=geometry.Vec3(0, 2.375, 2.375))
        hrrt_cbr1 = geometry.CubicRepeater(volume=hrrt_module.name, scale=geometry.Vec3(
            1, 9, 13), repeatVector=geometry.Vec3(0, 19.52, 19.52))
        hrrt_rr = geometry.RingRepeater(volume=hrrt_head.name, number=8)

        sys = camera.CylindericalPET()
        sys.attachSystem(itemList=['block', 'module', 'void', 'crystal'])
        camera1 = camera.Camera(name='cam1', system=sys)
        camera1.addGeo(hrrt_c1)
        camera1.addGeo(hrrt_cbr2)
        camera1.addGeo(hrrt_cbr1)
        camera1.addGeo(hrrt_rr)
        camera1.addCrystalSD(hrrt_crystal.name)
        ############################################################

        # phantom
        # c1 = geometry.Cylinder(
        #     mother='world', name='NEMACylinder', Rmax=82, Rmin=56, Height=5)
        # # print (b1.getMacStr())
        # # b1 = geometry.Box(mother=c1.name, name='', position=geometry.Vec3(
        # #     66.5, 0.0, 0.0), size=geometry.Vec3(20, 44, 5))
        # # c1.addChild(b1)

        # phantom = phantomModule.Phantom(name='phantom1')
        # phantom.addGeo(c1)
        # phantom.addPhantomSD(b1)

        pv1 = geometry.ImageRegularParamerisedVolume(
            mother='world', name='hof_heart', imagefile='heart_atn_phantom.h33', rangefile='range_atten_brain.dat', position=geometry.Vec3(0, 0, 0))
        phantom1 = phantom.Phantom(name='phantom1')
        phantom1.addGeo(pv1)
        phantom1.addPhantomSD(pv1)

        ##################################################

        # source
        src1 = source.VoxelizedSrcItem(name='voxel_heart')
        src1.addSrcModule(source.Voxelized(readtable='activity_range_brain.dat',
                                           readfile='heart_act_phantom.h33', position=geometry.Vec3(-38.4, -38.4, -0.15)))
        #src1 = source.SrcItem(name='src1')
        src1.addSrcModule(source.Particle(paticleType='gamma'))
        src1.addSrcModule(source.Angular(ang=[90, 90, 0, 360]))
        # src1.addSrcModule(Rectangle(halfSize = [10,20]))
        # src1.addSrcModule(source.Cylinder(
        #     dimension='Volume', halfz=10, radius=10))
        # src1.addSrcModule(source.Placement(
        #     placement=geometry.Vec3(10, 10, 10)))

        src = source.Source()
        src.addSourceItem(src1)
        ##################################################

        # digitizer
        sc = digitizer.SingleChain()
        a = digitizer.Adder()
        r = digitizer.Readout(depth=1)
        sc.addModule(a)
        sc.addModule(r)
        sc.addModule(digitizer.Blurring(res=0.10, eor=511))
        # sc.addModule(digitizer.CrystalBlurring())
        sc.addModule(digitizer.ThresHolder(holdvalue=250))
        sc.addModule(digitizer.UpHolder(holdvalue=750))
        # sc.addModule(digitizer.TimeResolution())
        # sc.addModule(digitizer.SpBlurring())
        sc.addModule(digitizer.DeadTime(dtVolume='block', deadtime=3000))
        coin1 = digitizer.CoinSorter(window=10, offset=0)
        coin2 = digitizer.CoinSorter(name='delay', window=10, offset=500)
        conichain1 = digitizer.CoinChain(
            name='finalcoin', inputList=[coin1.name, coin2.name], usePriority='true')
        # conichain1.addModule(digitizer.DeadTime(dtVolume='crystal'))
        # conichain1.addModule(digitizer.Buffer())
        digi = digitizer.Digitizer()
        digi.addModule(sc)
        digi.addModule(coin1)
        digi.addModule(coin2)
        digi.addModule(conichain1)
        #####################################################

        # physics
        phy = physics.Physics()
        phy.addCutPair(physics.CutPair(region='crystal', cutValue=0.1))
        phy.addCutPair(physics.CutPair(region=pv1.name, cutValue=0.1))
        phy.addMaxStep(physics.MaxStep(region=pv1.name, maxstepsize=0.01))

        simu = SimuApp(name=yml_filename, cam=camera1, phan=phantom1,
                       src=src, digi=digi, phy=phy, randEngine=RandomEngine())

        dataout1 = Root(fileName='testroot')
        dataout2 = Sino(fileName='testsino', inputDataName='finalcoin')

        simu.addDataout(dataout1)
        simu.addDataout(dataout2)
        # print(simu.getMacStr())
        # simu.generateMacs()
        simu.generateYaml()




if __name__ == '__main__':
    MacMaker.make_yml('test')




import yaml


class AttrPair:
    def __init__(self, value=None, fmtstr=None):
        self.value = value
        self.fmtstr = fmtstr


class DigiModule:
    moduleList = ['distributions', 'adder', 'adderCompton', 'readout',
                  'blurring', 'crystalBlurring', 'localBlurring', 'intrinsicResolutionBlurring ', 'quantumEfficiency ', 'calibration', 'crosstalk',
                  'thresholder', 'upholder', 'timeResolution', 'spBlurring', 'noise', 'localEffciency', 'buffer',
                  'deadtime']

    def __init__(self, moduleType=None, chainName=None):
        if moduleType in DigiModule.moduleList:
            self.moduleType = moduleType
        self.chainName = chainName
        self.attrList = []

    def makeAttrList(self):
        newFmt = r"/gate/digitizer/{0}/insert {1}" + "\n"
        self.addAttr(AttrPair(self.chainName, newFmt.format(
            self.chainName, self.moduleType)))

    def addAttr(self, attrItem):
        self.attrList.append(attrItem)

    def getMacStr(self):
        self.makeAttrList()
        mac = ""
        for item in self.attrList:
            if item.value is not None:
                mac += item.fmtstr
        return mac

    def getMeStr(self):
        fmt = r"/gate/digitizer/{0}/{1}"
        return fmt.format(self.chainName, self.moduleType)


# class Distributions(DigiModule):
#     def __init__(self):
#         pass


class Adder(DigiModule):
    def __init__(self, chainName=None):
        super(Adder, self).__init__(moduleType='adder', chainName=chainName)


class AdderCompton(DigiModule):
    def __init__(self, chainName=None):
        super(AdderCompton, self).__init__(
            moduleType='adderCompton', chainName=chainName)


class Readout(DigiModule):
    def __init__(self, chainName=None, policy='TakeEnergyCentroid', depth=1):
        super(Readout, self).__init__(
            moduleType='readout', chainName=chainName)
        self.policy = policy
        self.depth = depth

    def makeAttrList(self):
        super(Readout, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setPolicy {0} " + "\n"
        fmt2 = self.getMeStr() + r"/setDepth  {0} " + "\n"
        self.attrList.append(AttrPair(self.policy, fmt1.format(self.policy)))
        self.attrList.append(AttrPair(self.depth, fmt2.format(self.depth)))


class Blurring(DigiModule):
    def __init__(self, chainName=None, law = None, res=0.15, eor=511, slope=None):
        super(Blurring, self).__init__(
            moduleType='blurring', chainName=chainName)
        self.law = law
        self.resolution = res
        self.eor = eor
        self.slope = slope

    def makeAttrList(self):
        super(Blurring, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setLaw {0}" + "\n"
        fmt2 = self.getMeStr() + r"/setResolution {0}" + "\n"
        fmt3 = self.getMeStr() + r"/setEnergyOfReference {0} keV" + " \n"
        fmt4 = self.getMeStr() + r"/setSlope {0}  1/keV" + "\n"
        self.attrList.append(AttrPair(self.law, fmt1.format(self.law)))
        self.attrList.append(
            AttrPair(self.resolution, fmt2.format( self.resolution)))
        self.attrList.append(
            AttrPair(self.eor, fmt3.format( self.eor)))
        self.attrList.append(
            AttrPair(self.slope, fmt4.format( self.slope)))


class CrystalBlurring(DigiModule):
    def __init__(self, chainName=None, resWindow=[0, 1], qe=1, eor=511):
        super(CrystalBlurring, self).__init__(
            moduleType='crystalBlurring', chainName=chainName)
        self.resWindow = resWindow
        self.qe = qe
        self.eor = eor

    def makeAttrList(self):
        super(CrystalBlurring, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setCrystalResolutionMin {0}" + "\n" + self.getMeStr(
        ) + r"/setCrystalResolutionMax {1}" + "\n"
        fmt2 = self.getMeStr() + r"/setCrystalQE {0} " + " \n"
        fmt3 = self.getMeStr() + r"/setEnergyOfReference {0}  keV" + "\n"

        self.attrList.append(AttrPair(self.resWindow, fmt1.format(
            self.resWindow[0], self.resWindow[1])))
        self.attrList.append(AttrPair(self.qe, fmt2.format(self.qe)))
        self.attrList.append(AttrPair(self.eor, fmt3.format(self.eor)))


class ThresHolder(DigiModule):
    def __init__(self, chainName=None, holdvalue=250):
        super(ThresHolder, self).__init__(
            moduleType='thresholder', chainName=chainName)
        self.holdvalue = holdvalue

    def makeAttrList(self):
        super(ThresHolder, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setThreshold {0} keV" + "\n"
        self.attrList.append(
            AttrPair(self.holdvalue, fmt1.format(self.holdvalue)))


class UpHolder(DigiModule):
    def __init__(self, chainName=None,  holdvalue=750):
        super(UpHolder, self).__init__(
            moduleType='upholder', chainName=chainName)
        self.holdvalue = holdvalue

    def makeAttrList(self):
        super(UpHolder, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setUphold {0} keV" + "\n"
        self.attrList.append(
            AttrPair(self.holdvalue, fmt1.format(self.holdvalue)))


class TimeResolution(DigiModule):
    def __init__(self, chainName=None, res=1):
        super(TimeResolution, self).__init__(
            moduleType='timeResolution', chainName=chainName)
        self.resolution = res

    def makeAttrList(self):
        super(TimeResolution, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setTimeResolution {0}  ns" + "\n"
        self.attrList.append(
            AttrPair(self.resolution, fmt1.format(self.resolution)))


class SpBlurring(DigiModule):
    def __init__(self, chainName=None, res=2):
        super(SpBlurring, self).__init__(
            moduleType='spBlurring', chainName=chainName)
        self.resolution = res

    def makeAttrList(self):
        super(SpBlurring, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setSpresolution {0}  mm" + "\n"
        self.attrList.append(
            AttrPair(self.resolution, fmt1.format(self.resolution)))

class Buffer(DigiModule):
    def __init__(self, chainName = None, bufferSize =  None, readFreq = None, mode = None):
        super(Buffer, self).__init__(moduleType= 'buffer',chainName = chainName)
        self.bufferSize = bufferSize
        self.readFreq = readFreq
        self.mode = mode
    def makeAttrList(self):
        super(Buffer, self).makeAttrList()
        fmt1 = self.getMeStr() + r"/setBufferSize {0}  B" +"\n"
        fmt2 = self.getMeStr() + r"/setReadFrequency  {0}  MHz" +"\n"
        fmt3 = self.getMeStr() + r"/setMode {0}" +"\n"

        self.attrList.append(AttrPair(self.bufferSize,fmt1.format(self.bufferSize)))
        self.attrList.append(AttrPair(self.readFreq, fmt2.format(self.readFreq)))
        self.attrList.append(AttrPair(self.mode,fmt3.format(self.mode)))

class DeadTime(DigiModule):
    def __init__(self, dtVolume,deadtime, chainName=None, deadMode=None,  bufferSize=None, bufferMode=None):
        super(DeadTime, self).__init__(
            moduleType='deadtime', chainName=chainName)
        self.deadtime = deadtime
        self.deadMode = deadMode
        # if dtVolume is None:
        #     print("DeadTime: no Dead time volume")
        self.dtVolume = dtVolume
        self.bufferSize = bufferSize
        self.bufferMode = bufferMode

    def makeAttrList(self):
        super(DeadTime, self).makeAttrList()
        fmt0 = self.getMeStr() + r"/setDeadTime {0} ns" + "\n"
        fmt1 = self.getMeStr() + r"/setMode {0}" + "\n"
        fmt2 = self.getMeStr() + r"/chooseDTVolume {0}" + "\n"
        fmt3 = self.getMeStr() + r"/setBufferSize {0} MB" + "\n"
        fmt4 = self.getMeStr() + r"/setBufferMode {0} " + "\n"

        self.attrList.append(
            AttrPair(self.deadtime, fmt0.format(self.deadtime)))
        self.attrList.append(
            AttrPair(self.deadMode, fmt1.format(self.deadMode)))
        self.attrList.append(
            AttrPair(self.dtVolume, fmt2.format(self.dtVolume)))
        self.attrList.append(
            AttrPair(self.bufferSize, fmt3.format(self.bufferSize)))
        self.attrList.append(
            AttrPair(self.bufferMode, fmt4.format(self.bufferMode)))


class SingleChain:
    def __init__(self, name=None, input=None):
        if name is None:
            self.name = 'Singles'
            self.input = None
        else:
            self.name = name
            if input is None:
                self.input = 'Singles'
            else:
                self.input = input
        self.moduleList = []

    def addModule(self, item=None):
        if item is None:
            pass
        else:
            item.chainName = self.name
            self.moduleList.append(item)

    def getMacStr(self):
        mac = ""
        if self.name == 'Singles':
            pass
        else:
            fmt1 = r"/gate/digitizer/name {0}" + "\n"
            fmt2 = r"/gate/digitizer/insert SingleChain" + "\n"
            fmt3 = r"/gate/digitizer/{0}/setInputName {1}" + "\n"
            mac += fmt1.format(self.name) + fmt2.format() + \
                fmt3.format(self.name, self.input)
        for item in self.moduleList:
            mac += item.getMacStr()
        return mac





class CoinSorter:
    def __init__(self, name = None, inputName = None, window = None, minSectorDifference = None,
                 offset = None, depth = None, allPulsesOpenCoincGate = None, multipolicy = None):
        if name is None:
            self.name = 'Coincidences'
        else:
            self.name = name
        self.inputName = inputName
        self.window = window
        self.minSectorDifference = minSectorDifference
        self.offset = offset
        self.depth = depth
        self.allPulsesOpenCoincGate = allPulsesOpenCoincGate
        self.multipolicy = multipolicy
        self.attrList = []
   
    def makeAttrList(self):
        if self.name == 'Coincidences':
            pass
        else:            
            nameFmt = r"/gate/digitizer/name {0}" + "\n"
            insertFmt = r"/gate/digitizer/insert coincidenceSorter " + "\n"
            self.addAttr(AttrPair(self.name, nameFmt.format(self.name)+insertFmt.format()))
        fmt0 = self.getMeStr()+r"/addInputName {0}"
        fmt1 = self.getMeStr()+r"/setWindow {0}  ns" +"\n"
        fmt2 = self.getMeStr()+r"/minSectorDifference {0}" +"\n"
        fmt3 = self.getMeStr()+r"/setOffset {0}  ns" +"\n"
        fmt4 = self.getMeStr()+r"/setDepth {0}" +"\n"
        fmt5 = self.getMeStr()+r"/allPulsesOpenCoincGate {0}" + "\n"
        fmt6 = self.getMeStr()+r"/setMultiplePolicy {0}" + "\n"

        self.addAttr(AttrPair(self.inputName,fmt0.format(self.name,self.inputName)))
        self.addAttr(AttrPair(self.window,fmt1.format(self.window)))
        self.addAttr(AttrPair(self.minSectorDifference,fmt2.format(self.minSectorDifference)))
        self.addAttr(AttrPair(self.offset,fmt3.format(self.offset)))
        self.addAttr(AttrPair(self.depth,fmt4.format(self.depth)))
        self.addAttr(AttrPair(self.allPulsesOpenCoincGate,fmt5.format(self.allPulsesOpenCoincGate)))
        self.addAttr(AttrPair(self.multipolicy,fmt6.format(self.multipolicy)))        

    def addAttr(self, attrItem):
        self.attrList.append(attrItem)

    def getMacStr(self):
        self.makeAttrList()
        mac = ""
        for item in self.attrList:
            if item.value is not None:
                mac += item.fmtstr
        return mac

    def getMeStr(self):
        fmt = r"/gate/digitizer/{0}"
        return fmt.format(self.name)


class CoinChain:
    def __init__(self, name = None, inputList = None, usePriority = None):
        if name is None:
            print("No CoinChain name! CoinChain:__init__() \n")
        else:
            self.name = name
        if inputList is None:
            self.inputList = []
        else:
            self.inputList = inputList
        self.moduleList = []
        self.usePriority = usePriority
    def addInput(self, item):
        if item is not None:
            self.inputList.append(item)
   

    def addModule(self, item=None):
        if item is not None:
            item.chainName = self.name
            self.moduleList.append(item)

    def getMacStr(self):
        mac = ""
        if self.name is None:
            pass
        else:
            fmt1 = r"/gate/digitizer/name {0}" + "\n"
            fmt2 = r"/gate/digitizer/insert coincidenceChain" + "\n"
            mac += fmt1.format(self.name) + fmt2.format()
        fmt3 = r"/gate/digitizer/{0}/addInputName {1}" + "\n"
        for item in self.inputList:
            mac += fmt3.format(self.name,item)     
        for item in self.moduleList:
            mac += item.getMacStr()
        if self.usePriority is not None:
            fmt4 = r"/gate/digitizer/{0}/usePriority true"
            mac += fmt4.format(self.name)
        return mac


class Digitizer:
    def __init__(self):
        self.moduleList = []
    
    def addModule(self, module):
        self.moduleList.append(module)

    def getMacStr(self):
        mac =""
        for item in self.moduleList:
            mac += item.getMacStr()
        return mac


# if __name__ == '__main__':


#     # print(digi.getMacStr())

# with open('digitizer.yml', 'w') as fout:
#     yaml.dump(digi, fout)



from .digitizer import AttrPair
from .geometry import Vec3
import yaml


class SrcModule:
    def __init__(self, srcName=None):
        self.srcName = srcName
        self.attrList = []

    def getMacStr(self):
        self.makeAttrList()
        mac = ""
        for item in self.attrList:
            if item.value is not None:
                mac += item.fmtstr
        return mac

    def makeAttrList(self):
        pass

    def addAttr(self, attrItem):
        self.attrList.append(attrItem)


class Particle(SrcModule):
    def __init__(self, srcName=None, paticleType='positron'):
        super(Particle, self).__init__(srcName=srcName)
        self.particleType = paticleType

    def makeAttrList(self):
        if self.particleType == 'positron':
            fmt = (r"/gate/source/{0}/gps/particle e+ " + "\n" +
                   r"/gate/source/{0}/gps/energytype Fluor18" + "\n" +
                   r"/gate/source/{0}/setForcedUnstableFlag true" + "\n" +
                   r"/gate/source/{0}/setForcedHalfLife 6586 s" + "\n")
            self.addAttr(AttrPair(self.particleType, fmt.format(self.srcName)))
        elif self.particleType == 'gamma':
            fmt = (r"/gate/source/{0}/setType backtoback" + "\n" +
                   r"/gate/source/{0}/gps/particle gamma " + "\n" +
                   r"/gate/source/{0}/gps/monoenergy 511 keV" + "\n" +
                   r"/gate/source/{0}/setForcedUnstableFlag true" + "\n" +
                   r"/gate/source/{0}/setForcedHalfLife 6586.2 s" + "\n")
            self.addAttr(AttrPair(self.particleType, fmt.format(self.srcName)))
        else:
            print("invalid particle type in Particle:makeAttrList() \n")


class Activity(SrcModule):
    def __init__(self, srcName=None, activity=None):
        super(Activity, self).__init__(srcName=srcName)
        self.activity = activity

    def makeAttrList(self):
        fmt = r"/gate/source/{0}/setActivity {1}  becquerel" + "\n"
        self.addAttr(AttrPair(self.activity, fmt.format(
            self.srcName, self.activity)))


class Angular(SrcModule):
    def __init__(self, srcName=None, ang=[0, 180, 0, 360]):
        super(Angular, self).__init__(srcName=srcName)
        self.ang = ang

    def makeAttrList(self):
        fmt = (r"/gate/source/{0}/gps/angtype iso" + "\n" +
               r"/gate/source/{0}/gps/mintheta {1} deg " + "\n" +
               r"/gate/source/{0}/gps/maxtheta {2} deg" + "\n" +
               r"/gate/source/{0}/gps/minphi   {3} deg" + "\n" +
               r"/gate/source/{0}/gps/maxphi   {4} deg" + "\n")
        self.addAttr(AttrPair(self.ang, fmt.format(
            self.srcName, self.ang[0], self.ang[1], self.ang[2], self.ang[3])))


class Voxelized(SrcModule):
    def __init__(self, readtable, readfile, shape='Voxelized', reader='interfile', translator='range', srcName=None, position=None):
        super(Voxelized, self).__init__(srcName=srcName)
        self.shape = shape
        self.reader = reader
        self.translator = translator
        self.readtable = readtable
        self.readfile = readfile
        self.position = position

    def makeAttrList(self):
        fmt = (r"/gate/source/{0}/reader/insert {1}" + "\n" +
               r"/gate/source/{0}/{1}Reader/translator/insert {2} " + "\n" +
               r"/gate/source/{0}/{1}Reader/{2}Translator/readTable {3}" + "\n" +
               r"/gate/source/{0}/{1}Reader/readFile  {4}" + "\n" +
               r"/gate/source/{0}/setPosition  {5}  mm" + "\n")
        self.addAttr(AttrPair(self.shape, fmt.format(self.srcName, self.reader,
                                                     self.translator, self.readtable, self.readfile, self.position.getMacStr())))


class Shape(SrcModule):
    PlaneList = ['Circle', 'Annulus', 'Ellpsoid', 'Square', 'Rectangle']
    VolumeList = ['Sphere', 'Ellipsoid', 'Cylinder', 'Para']

    def __init__(self, dimension, shape, srcName=None):
        super(Shape, self).__init__(srcName=srcName)
        if dimension is 'Plane':
            if shape in Shape.PlaneList:
                self.dimension = dimension
                self.shape = shape
        elif ((dimension is 'Surface') or (dimension is 'Volume')):
            if shape in Shape.VolumeList:
                self.dimension = dimension
                self.shape = shape
        else:
            pass

    def makeAttrList(self):
        fmt1 = r"/gate/source/{0}/gps/type {1}" + "\n"
        fmt2 = r"/gate/source/{0}/gps/shape {1}" + "\n"
        self.addAttr(AttrPair(self.dimension, fmt1.format(
            self.srcName, self.dimension)))
        self.addAttr(
            AttrPair(self.shape, fmt2.format(self.srcName, self.shape)))


class Cylinder(Shape):
    def __init__(self, radius, halfz, dimension, srcName=None):
        super(Cylinder, self).__init__(
            dimension=dimension, shape='Cylinder', srcName=srcName)
        self.radius = radius
        self.halfz = halfz

    def makeAttrList(self):
        super(Cylinder, self).makeAttrList()
        fmt1 = r"/gate/source/{0}/gps/radius {1} mm" + "\n"
        fmt2 = r"/gate/source/{0}/gps/halfz {1} mm" + "\n"
        self.addAttr(
            AttrPair(self.radius, fmt1.format(self.srcName, self.radius)))
        self.addAttr(
            AttrPair(self.halfz, fmt2.format(self.srcName, self.halfz)))


class Sphere(Shape):
    def __init__(self, radius, dimension, srcName=None):
        super(Sphere, self).__init__(
            dimension=dimension, shape='Sphere', srcName=srcName)
        self.radius = radius

    def makeAttrList(self):
        super(Sphere, self).makeAttrList()
        fmt1 = r"/gate/source/{0}/gps/radius {1} mm" + "\n"
        self.addAttr(
            AttrPair(self.radius, fmt1.format(self.srcName, self.radius)))


class Ellipsoid(Shape):
    def __init__(self, halfSize, dimension, srcName=None):
        super(Ellipsoid, self).__init__(
            dimension=dimension, shape='Ellipsoid', srcName=srcName)
        self.halfSize = halfSize

    def makeAttrList(self):
        super(Ellipsoid, self).makeAttrList()
        fmt1 = (r"/gate/source/{0}/gps/halfx {1} mm" + "\n" +
                r"/gate/source/{0}/gps/halfy {2} mm" + "\n" +
                r"/gate/source/{0}/gps/halfz {3} mm" + "\n")
        self.addAttr(AttrPair(self.halfSize, fmt1.format(
            self.srcName, self.halfSize[0], self.halfSize[1], self.halfSize[2])))


class Circle(Shape):
    def __init__(self, radius, srcName=None):
        super(Circle, self).__init__(
            dimension='Plane', shape='Circle', srcName=srcName)
        self.radius = radius

    def makeAttrList(self):
        super(Circle, self).makeAttrList()
        fmt1 = (r"/gate/source/{0}/gps/radius {1} mm" + "\n")
        self.addAttr(
            AttrPair(self.radius, fmt1.format(self.srcName, self.radius)))


class Annulus(Shape):
    def __init__(self, radius0, radius, srcName=None):
        super(Annulus, self).__init__(
            dimension='Plane', shape='Annulus', srcName=srcName)
        self.radius = radius
        self.radius0 = radius0

    def makeAttrList(self):
        super(Annulus, self).makeAttrList()
        fmt1 = (r"/gate/source/{0}/gps/radius0 {1} mm" + "\n")
        fmt2 = (r"/gate/source/{0}/gps/radius {1} mm" + "\n")
        self.addAttr(
            AttrPair(self.radius0, fmt1.format(self.srcName, self.radius0)))
        self.addAttr(
            AttrPair(self.radius, fmt2.format(self.srcName, self.radius)))


class Ellipse(Shape):
    def __init__(self, halfSize, srcName=None):
        super(Ellipse, self).__init__(
            dimension="Plane", shape='Ellipse', srcName=srcName)
        self.halfSize = halfSize

    def makeAttrList(self):
        super(Ellipse, self).makeAttrList()
        fmt1 = (r"/gate/source/{0}/gps/halfx {1} mm" + "\n" +
                r"/gate/source/{0}/gps/halfy {2} mm" + "\n")
        self.addAttr(AttrPair(self.halfSize, fmt1.format(
            self.srcName, self.halfSize[0], self.halfSize[1])))


class Rectangle(Shape):
    def __init__(self, halfSize, srcName=None):
        super(Rectangle, self).__init__(
            dimension="Plane", shape='Rectangle', srcName=srcName)
        self.halfSize = halfSize

    def makeAttrList(self):
        super(Rectangle, self).makeAttrList()
        fmt1 = (r"/gate/source/{0}/gps/halfx {1} mm" + "\n" +
                r"/gate/source/{0}/gps/halfy {2} mm" + "\n")
        self.addAttr(AttrPair(self.halfSize, fmt1.format(
            self.srcName, self.halfSize[0], self.halfSize[1])))


class Placement(SrcModule):
    def __init__(self, srcName=None, placement=None):
        super(Placement, self).__init__(srcName=srcName)
        self.placement = placement

    def makeAttrList(self):
        fmt = r"/gate/source/{0}/centre {1}  mm" + "\n"
        self.addAttr(AttrPair(self.placement, fmt.format(
            self.srcName, self.placement.getMacStr())))


class SrcItem:
    def __init__(self, name):
        self.srcModuleList = []
        self.name = name

    def addSrcModule(self, item):
        item.srcName = self.name
        self.srcModuleList.append(item)

    def getMacStr(self):
        mac = ""
        fmt = r"/gate/source/addSource {0}" + "\n"
        mac += fmt.format(self.name)
        for item in self.srcModuleList:
            mac += item.getMacStr()
        return mac


class VoxelizedSrcItem:
    def __init__(self, name):
        self.srcModuleList = []
        self.name = name

    def addSrcModule(self, item):
        item.srcName = self.name
        self.srcModuleList.append(item)

    def getMacStr(self):
        mac = ""
        fmt = r"/gate/source/addSource {0} voxel" + "\n"
        mac += fmt.format(self.name)
        for item in self.srcModuleList:
            mac += item.getMacStr()
        return mac


class Source():
    def __init__(self):
        self.srcItemList = []

    def addSourceItem(self, item):
        self.srcItemList.append(item)

    def getMacStr(self):
        mac = ""
        for item in self.srcItemList:
            mac += item.getMacStr()
        mac += r"/gate/source/list" + "\n"
        return mac


# if __name__ is "__main__":

#     src1 = SrcItem(name='src1')
#     src1.addSrcModule(Particle(paticleType='gamma'))
#     src1.addSrcModule(Angular(ang=[90, 90, 0, 360]))
#     # src1.addSrcModule(Rectangle(halfSize = [10,20]))
#     src1.addSrcModule(Cylinder(dimension = 'Volume',halfz = 10 , radius = 10))
#     src1.addSrcModule(Placement(placement=Vec3(10, 10, 10)))

#     src = Source()
#     src.addSourceItem(src1)

#     print(src.getMacStr())
# with open('source.yml', 'w') as fout:
#     yaml.dump(src, fout)



from setuptools import setup, find_packages

setup(name='dxl-pygate',
      version='0.10.8',
      description='A simplified python interface for Gate.',
      url='https://github.com/Hong-Xiang/pygate',
      author='Hong Xiang',
      author_email='hx.hongxiang@gmail.com',
      license='MIT',
      packages=find_packages(),
      install_requires=['fs', 'click', 'pyyaml',
                        'rx', 'jinja2', 'dxl-dxpy>=0.5'],
      scripts=['bin/pygate'],
      include_package_data=True,
      zip_safe=False)



import yaml

class CutPair:
    def __init__(self, region=None, cutValue=None):
        self.cutValue = cutValue
        self.region = region
class MaxStep:
    def __init__(self,region = None, maxstepsize = None):
        self.maxstepsize = maxstepsize
        self.region = region
class Physics():
    def __init__(self):
        # self.flag = True
        # self.CutStr = ""
        self.cutPairList = []
        self.maxstepsizeList = []
    def getModelStr(self):
        return(r"/gate/physics/addProcess PhotoElectric" + "\n" 
                +"/gate/physics/processes/PhotoElectric/setModel StandardModel" + "\n" 
                +r"/gate/physics/addProcess Compton" + "\n" 
                +r"/gate/physics/processes/Compton/setModel StandardModel" + "\n" 
                +r"/gate/physics/addProcess RayleighScattering" + "\n" 
                +r"/gate/physics/processes/RayleighScattering/setModel PenelopeModel" + "\n" 
                +r"/gate/physics/addProcess ElectronIonisation" + "\n" 
                +r"/gate/physics/processes/ElectronIonisation/setModel StandardModel e-" + "\n" 
                +r"/gate/physics/processes/ElectronIonisation/setModel StandardModel e+" + "\n" 
                +r"/gate/physics/addProcess Bremsstrahlung" + "\n" 
                +r"/gate/physics/processes/Bremsstrahlung/setModel StandardModel e-" + "\n" 
                +r"/gate/physics/processes/Bremsstrahlung/setModel StandardModel e+" + "\n" 
                +r"/gate/physics/addProcess PositronAnnihilation" + "\n" 
                +r"/gate/physics/addProcess MultipleScattering e+"+"\n"
                +r"/gate/physics/addProcess MultipleScattering e-"+"\n"
               # +r"/gate/physics/addProcess eMultipleScattering" + "\n" 
               # +r"/gate/physics/processes/eMultipleScattering/setGeometricalStepLimiterType e- distanceToBoundary" + "\n" 
               # +r"/gate/physics/processes/eMultipleScattering/setGeometricalStepLimiterType e+ distanceToBoundary" + "\n" 
               # +r"/gate/physics/addProcess RadioactiveDecay" + "\n" 
               # +r"/gate/physics/addAtomDeexcitation" + "\n" 
                +r"/gate/physics/processList Enabled" + "\n" 
                +r"/gate/physics/processList Initialized" + "\n" )

    def addCutPair(self,item):
        if item.cutValue is None or item.region is None:
            pass
        else:
            self.cutPairList.append(item)

    def addMaxStep(self,item):
        if item.maxstepsize is None or item.region is None:
            pass
        else:
            self.maxstepsizeList.append(item)

    def getMacStr(self):
        fmt1 = r"/gate/physics/Gamma/SetCutInRegion {0}  {1}"+" mm\n"
        fmt2 = r"/gate/physics/Electron/SetCutInRegion {0}  {1}"+" mm\n"
        fmt3 = r"/gate/physics/Positron/SetCutInRegion {0}  {1}"+" mm\n"

        fmt4 = r"/gate/physics/SetMaxStepSizeInRegion {0} {1}" +" mm\n"
        Str = ""
        for item in self.cutPairList:
            Str += fmt1.format(item.region,item.cutValue)
            Str += fmt2.format(item.region,item.cutValue)
            Str += fmt3.format(item.region,item.cutValue)
        for item in self.maxstepsizeList:
            Str += fmt4.format(item.region,item.maxstepsize)
        return (self.getModelStr()+Str)

# if __name__ == '__main__':
#     phy = Physics()
#     phy.addCutPair(CutPair(region = 'level1', cutValue = 10 ))
#     print(phy.getMacStr())
#     with open('physics.yml', 'w') as fout:
#         yaml.dump(phy, fout)

        


